; ##############################################################################
; ##############################################################################
; ##############################################################################
ROM_COMMAND_AUSWERTUNG:
	; ROM Command schon komplett abgearbeitet?		
	LDS temp,(adr_OW_RX_ROM_COMMAND_FERTIG)
	tst temp
	breq ROM_COMMAND_AUSWERTUNG_RUN
	ret
ROM_COMMAND_AUSWERTUNG_RUN:
	rcall ROM_SEARCH
	rcall ROM_MATCH
	rcall ROM_READ
	rcall ROM_COMMAND_RX
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ROM COMMAND bitweise einlesen													
ROM_COMMAND_RX:
	; ROM Command schon erkannt / noch NULL ?
	LDS temp,(adr_OW_RX_ROM_COMMAND)
	tst temp
	breq ROM_COMMAND_RX_RUN
	ret
; ------------------------------------------------------------------------------
ROM_COMMAND_RX_RUN:
	; 0/1 SLOT ?				
	rcall STATUS_ABFRAGE_UMWANDLUNG_OUT_TEMP ; OUT: 0=0SLOT 1=1SLOT
	tst temp
	brne ROM_COMMAND_RX_1SLOT	; RX 1-SLOT						
; ------------------------------------------------------------------------------
; 0-SLOT wurde empfangen														
ROM_COMMAND_RX_0SLOT:			; RX 0-SLOT						
	LDS temp,(adr_OW_RX_BYTE)	; "BITs" laden					
	LSR temp					; 1xrechts ohne CARRY			
	STS(adr_OW_RX_BYTE),temp	; "BITs" speichern				
	rjmp ROM_COMMAND_RX_SLOT
; ------------------------------------------------------------------------------
; 1-SLOT wurde empfangen														
ROM_COMMAND_RX_1SLOT:			; RX 1-SLOT						
	LDS temp,(adr_OW_RX_BYTE)	; "BITs" laden					
	SEC							; C=1							
	ROR temp					; 1xrechts ber Carry 			
	STS(adr_OW_RX_BYTE),temp	; "BITs" speichern				
; ------------------------------------------------------------------------------
; SLOT auswerten																
ROM_COMMAND_RX_SLOT:	
	rcall FUNKTION_BIT_ZAEHLER_PLUS_EINS
	LDS temp,(adr_OW_BIT_ZAEHLER)	; BIT-ZHLER laden			
	cpi temp,8						; vergleich					
	breq ROM_COMMAND_RX_SLOT_BYTE
	ret
; ------------------------------------------------------------------------------
; BYTE auswerten																
ROM_COMMAND_RX_SLOT_BYTE:
	; "BITs" laden					
	LDS temp,(adr_OW_RX_BYTE)
	; ROM-COMMAND speichern SRAM	
	STS(adr_OW_RX_ROM_COMMAND),temp
	; NULL							
	STS(adr_OW_BIT_ZAEHLER),NULL		; BIT-ZEIGER NULL		
	STS(adr_OW_RX_BYTE),NULL			; "BITs" NULL			
	STS(adr_OW_ZEIGER),NULL				; ZEIGER				
	STS(adr_OW_ROM_SEARCH_ZAEHLER),NULL	; SEARCH-ZAEHLER		
	STS(adr_OW_STATUS),NULL				; STATUS				
; ------------------------------------------------------------------------------
	; AUSWERTUNG					
	cpi temp,(OW_COMMAND_READ_ROM)	; READ	
	breq ROM_COMMAND_RX_READ_ROM
	cpi temp,(OW_COMMAND_MATCH_ROM)	; MATCH	
	breq ROM_COMMAND_RX_MATCH_ROM
	cpi temp,(OW_COMMAND_SEARCH_ROM); SEARCH
	breq ROM_COMMAND_RX_SEARCH_ROM
	cpi temp,(OW_COMMAND_SKIP_ROM)	; SKIP	
	breq ROM_COMMAND_RX_SKIP_ROM
	; unbekanntes ROM COMMAND ==> SPERREN
	rjmp STATUS_SPERREN_SETZEN		;  STATUS: gesperrt	
; ------------------------------------------------------------------------------
ROM_COMMAND_RX_READ_ROM:
	rjmp ROM_READ_START
; ------------------------------------------------------------------------------
ROM_COMMAND_RX_MATCH_ROM:
	rjmp ROM_MATCH_START
; ------------------------------------------------------------------------------
ROM_COMMAND_RX_SEARCH_ROM:
	rjmp ROM_SEARCH_START
; ------------------------------------------------------------------------------
ROM_COMMAND_RX_SKIP_ROM:
	STS(adr_OW_RX_ROM_COMMAND_FERTIG),EINS	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ROM_READ_START:

	rjmp ROM_READ
; ##############################################################################
; ##############################################################################
; ##############################################################################
ROM_READ:
	; ROM READ ?	
	LDS temp,(adr_OW_RX_ROM_COMMAND)	
	cpi temp,(OW_COMMAND_READ_ROM)
	breq ROM_READ_RUN
	ret
; ------------------------------------------------------------------------------
ROM_READ_RUN:
	; ZEIGER			
	ldi ZL, LOW(adr_OW_ROM_ID_BEREICH8+0)
	ldi ZH,HIGH(adr_OW_ROM_ID_BEREICH8+0)
	; BIT-Zhler		
	LDS temp,(adr_OW_BIT_ZAEHLER)
	; BIT auslesen		
	rcall FUNKTION_SRAM_BIT_READ_Z_TEMP	; inp: temp+Z OUT: temp+Carry	
	brcc ROM_READ_WRITE_0SLOT			; CARRY=0 ==> SPRUNG			
ROM_READ_WRITE_1SLOT:
	; STATUS			
	rcall STATUS_TX_1SLOT_SETZEN ; STATUS: TX-1SLOT
	rjmp FUNKTION_BIT_ZAEHLER_PLUS_EINS
ROM_READ_WRITE_0SLOT:
	; STATUS			
	rcall STATUS_TX_0SLOT_SETZEN ; STATUS: TX-0SLOT
	rjmp FUNKTION_BIT_ZAEHLER_PLUS_EINS
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
ROM_MATCH_START:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
ROM_MATCH:
	; ROM MATCH ?	
	LDS temp,(adr_OW_RX_ROM_COMMAND)
	cpi temp,(OW_COMMAND_MATCH_ROM)
	breq ROM_MATCH_RUN
	ret
; ------------------------------------------------------------------------------
ROM_MATCH_RUN:
	; STATUS laden und abfragen
	rcall STATUS_ABFRAGE_UMWANDLUNG_OUT_TEMP ; OUT: 0=0SLOT 1=1SLOT	
	mov temp1,temp
; ------------------------------------------------------------------------------
	; Z-ZEIGER			
	ldi ZL, LOW(adr_OW_ROM_ID_BEREICH8)
	ldi ZH,HIGH(adr_OW_ROM_ID_BEREICH8)
	; BIT-Zhler		
	LDS temp,(adr_OW_BIT_ZAEHLER)
	; BIT auslesen		
	rcall FUNKTION_SRAM_BIT_READ_Z_TEMP	; inp: temp+Z OUT: temp+Carry	
; ------------------------------------------------------------------------------
	; Vergleich	(empfangenes Bit + Bit ROM-ID)	
	cp temp,temp1
	breq ROM_MATCH_OK
	; SPERREN						
	rjmp STATUS_SPERREN_SETZEN	;  STATUS: gesperrt
; ------------------------------------------------------------------------------
ROM_MATCH_OK:
	; BIT-Zhler+1	
	rcall FUNKTION_BIT_ZAEHLER_PLUS_EINS
	; FERTIG ?	
	LDS temp,(adr_OW_BIT_ZAEHLER)
	cpi temp,64
	breq ROM_MATCH_FERTIG
	ret
; ------------------------------------------------------------------------------
ROM_MATCH_FERTIG:
	STS(adr_OW_RX_ROM_COMMAND_FERTIG),EINS
	STS(adr_OW_BIT_ZAEHLER),NULL
	STS(adr_OW_ZEIGER),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
ROM_SEARCH_START:

rjmp ROM_SEARCH
; ##############################################################################
; ##############################################################################
; ##############################################################################
; INP: 	SRAM(adr_OW_STATUS)														
ROM_SEARCH:
	; ROM SEARCH ?	
	LDS temp,(adr_OW_RX_ROM_COMMAND)
	cpi temp,(OW_COMMAND_SEARCH_ROM)
	breq ROM_SEARCH_RUN
ret
; ------------------------------------------------------------------------------
; ZHLER=1=A=TX ROMBIT															
; ZHLER=2=B=TX ROMBIT (Komplement)												
; ZHLER=3=  RX ANTWORT MASTER													
ROM_SEARCH_RUN:
	; maximal 64 A/B BITS senden	
	LDS temp,(adr_OW_BIT_ZAEHLER)
	cpi temp,64+1
	brlo ROM_SEARCH_ACTION
	; SPERREN						
	rjmp STATUS_SPERREN_SETZEN	;  STATUS: gesperrt
; ------------------------------------------------------------------------------
ROM_SEARCH_ACTION:
	; SEARCH ZHLER+1		
	LDS temp,(adr_OW_ROM_SEARCH_ZAEHLER)
	inc temp
	STS (adr_OW_ROM_SEARCH_ZAEHLER),temp
	; ZHLER Vergleich				
	cpi temp,1		; ZHLER=1	
	breq ROM_SEARCH_WRITE_A
	cpi temp,2		; ZHLER=2	
	breq ROM_SEARCH_WRITE_B
	cpi temp,4
	breq ROM_SEARCH_VERGLEICH
	ret
; ------------------------------------------------------------------------------
ROM_SEARCH_VERGLEICH:
	; SEARCH ZHLER				
	STS(adr_OW_ROM_SEARCH_ZAEHLER),NULL	; ZHLER 
	; STATUS laden und abfragen
	rcall STATUS_ABFRAGE_UMWANDLUNG_OUT_TEMP ; OUT: 0=0SLOT 1=1SLOT	
	; A laden 						
	LDS temp1,(adr_OW_ROM_SEARCH_A)
	; VERGLEICH A und RX-BIT		
	cp temp, temp1
	breq ROM_SEARCH_ACTION
	; SPERREN						
	rjmp STATUS_SPERREN_SETZEN	;  STATUS: gesperrt
; ------------------------------------------------------------------------------
ROM_SEARCH_WRITE_A:
	; Z-ZEIGER			
	ldi ZL, LOW(adr_OW_ROM_ID_BEREICH8)
	ldi ZH,HIGH(adr_OW_ROM_ID_BEREICH8)
	; BIT-Zhler		
	rcall FUNKTION_BIT_ZAEHLER_PLUS_EINS
	LDS temp,(adr_OW_BIT_ZAEHLER)
	dec temp
	; BIT auslesen		
	rcall FUNKTION_SRAM_BIT_READ_Z_TEMP	; inp: temp+Z OUT: temp+Carry	
	; A speichern		
	STS(adr_OW_ROM_SEARCH_A),temp
	; A senden			
	tst temp
	breq ROM_SEARCH_WRITE_0SLOT
	rjmp ROM_SEARCH_WRITE_1SLOT
; ------------------------------------------------------------------------------
; Komplement																	
ROM_SEARCH_WRITE_B:
	; A laden und Komplement bilden	
	LDS temp,(adr_OW_ROM_SEARCH_A)
	; B senden						
	tst temp
	breq ROM_SEARCH_WRITE_1SLOT	; invertieren	
	rjmp ROM_SEARCH_WRITE_0SLOT
; ------------------------------------------------------------------------------
ROM_SEARCH_WRITE_0SLOT:
	; STATUS			
	rjmp STATUS_TX_0SLOT_SETZEN ; STATUS: TX-0SLOT
ROM_SEARCH_WRITE_1SLOT:
	; STATUS			
	rjmp STATUS_TX_1SLOT_SETZEN ; STATUS: TX-1SLOT
; ##############################################################################
; ##############################################################################
; ##############################################################################

