;												
; 1-WIRE-SLAVE (simuliert den DS19s20)			
; ATmega8										
; Bernhard.Erfurt@gmx.de						
; 06/2012										
;												
; LCD 16x2										
;												

;												
; Anschlussbelegung PINS						
;												
; PB0 ==> LCD									
; PB1 ==> LCD									
; PB2 ==> LCD									
; PB3 ==> LCD									
; PB4 ==> LCD R/S								
; PB5 ==> LCD E									
; PB6 ==> Quarz									
; PB7 ==> Quarz									
;												
; PC0 ==> 										
; PC1 ==> 										
; PC2 ==> 										
; PC3 ==> 										
; PC4 ==> 										
; PC5 ==> 										
;												
; PD0 ==> TRIGGER-AUSGANG (fr WARTUNGSZWECKE)	
; PD1 ==> 										
; PD2 ==> 1-WIRE								
; PD3 ==> 										
; PD4 ==> 										
; PD5 ==> LED GRN	1-WIRE SLAVE RX				
; PD6 ==> LED GELB 	1-WIRE SLAVE TX				
; PD7 ==> LED ROT	1-WIRE(ERROR)				
;												


; ATmega8 										
.include "m8def.inc"
; Programm-VERSION								
.equ VERSION_NR					= 6
.equ VERSION_TAG				= 15
.equ VERSION_MONAT				= 6
.equ VERSION_JAHR				= 2012
; KONSTANTEN									
.equ TAKT						=16000000			; Hz	
; 1-WIRE-ROM-ID	(Prfsumme wird automatisch korrigiert)		
.equ ROM_ID_BYTE1				= 0x10	; FAMILY x10=DS18s20 x27=DS2417		
.equ ROM_ID_BYTE2				= 0x00
.equ ROM_ID_BYTE3				= 0x18
.equ ROM_ID_BYTE4				= 0x06
.equ ROM_ID_BYTE5				= 0x12
.equ ROM_ID_BYTE6				= 0x20
.equ ROM_ID_BYTE7				= 0x00
.equ ROM_ID_BYTE8				= 0x00	; Prfsumme
; 1-WIRE-ERROR-CODES							
.equ ERROR_FUNCTION_COMAND		= 0xF0
; 1-WIRE-ROM-COMMANDS							
.equ OW_COMMAND_READ_ROM		= 0x33
.equ OW_COMMAND_MATCH_ROM		= 0x55
.equ OW_COMMAND_SEARCH_ROM		= 0xF0
.equ OW_COMMAND_SKIP_ROM		= 0xCC
; 1-WIRE-FUNCTION-COMMANDS						
.equ OW_FUNCTION_READ_SCRATCHPAD_A	=0xBE	;0xBE= DS19s20
.equ OW_FUNCTION_WRITE_SCRATCHPAD_A	=0x4E	;0x4E= DS19s20
.equ OW_FUNCTION_READ_SCRATCHPAD_B	=0x66
.equ OW_FUNCTION_WRITE_SCRATCHPAD_B	=0x99
.equ OW_FUNCTION_READ_SCRATCHPAD_C	=0xBC
.equ OW_FUNCTION_WRITE_SCRATCHPAD_C	=0x4C
; 1-WIRE-FUNCTION-SONDER-COMMANDS 				
; (z.B. fr Start einer Messung)				
.equ OW_FUNCTION_SONDER_A			=0x44
.equ OW_FUNCTION_SONDER_B			=0x55
.equ OW_FUNCTION_SONDER_C			=0x66
; 1-WIRE-STATUS									
.equ OW_STATUS_RX_BEREIT	=0x00			; Empfangsbereit								
.equ OW_STATUS_RX_0SLOT		=0xE0			; eine "0" wurde vom MASTER empfangen			
.equ OW_STATUS_RX_1SLOT		=0xE1			; eine "1" wurde vom MASTER empfangen			
.equ OW_STATUS_TX_0SLOT		=0xF0			; eine "0" muss zum MASTER gesendet werden		
.equ OW_STATUS_TX_1SLOT		=0xF1			; eine "1" muss zum MASTER gesendet werden		
.equ OW_STATUS_GESPERRT		=0xFF			; GESPERRT										
; 1-WIRE-DATENBUS								
.equ OW_PORT				= PORTD
.equ OW_DDR					= DDRD
.equ OW_PIN					= PIND
.equ OW_PORTBIT				= 2
; 1-WIRE LED-PORT 								
.equ OW_LED_PORT			= PORTD
.equ OW_LED_DDR				= DDRD
; 1-WIRE LEDS 									
.equ LED_OW_GRUEN			= 5
.equ LED_OW_GELB			= 6
.equ LED_OW_ROT				= 7
.equ LED_OW_TRIGGER			= 0	; fr Testzwecke
; LCD (DATA)									
.equ LCD_DATA0_DDR			= DDRB
.equ LCD_DATA1_DDR			= DDRB
.equ LCD_DATA2_DDR			= DDRB
.equ LCD_DATA3_DDR			= DDRB
.equ LCD_DATA0_PORT			= PORTB
.equ LCD_DATA1_PORT			= PORTB
.equ LCD_DATA2_PORT			= PORTB
.equ LCD_DATA3_PORT			= PORTB
.equ LCD_DATA0				= 0
.equ LCD_DATA1				= 1
.equ LCD_DATA2				= 2
.equ LCD_DATA3				= 3
; LCD RS (Befehl/DATEN)							
.equ LCD_RS_DDR				= DDRB
.equ LCD_RS_PORT			= PORTB
.equ LCD_RS					= 4
; LCD_ENABLE									
.equ LCD_E_DDR				= DDRB
.equ LCD_E_PORT				= PORTB
.equ LCD_E					= 5
; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						=0x0060			; 0x0060 START Adresse 		



.equ adr_OW_STATUS					=0x0060+600		; OW-STATUS						
.equ adr_OW_ERROR_CODE				=0x0060+602		; ERROR CODE vom ersten auftretenden Fehler		
.equ adr_OW_ERROR_WERT				=0x0060+604		; WERT						
.equ adr_OW_ZAEHLER_RESET_PRESENCE_L=0x0060+606		; fr Wartungszwecke (ZAEHLERWERTE)
.equ adr_OW_ZAEHLER_RESET_PRESENCE_H=0x0060+608		; fr Wartungszwecke (ZAEHLERWERTE)
.equ adr_OW_BIT_ZAEHLER				=0x0060+610		; BIT-ZHLER					
.equ adr_OW_ZEIGER					=0x0060+612		; ZEIGER						
.equ adr_OW_RX_BYTE					=0x0060+614		; RX-Byte						
.equ adr_OW_ROM_SEARCH_ZAEHLER		=0x0060+616		; Zhler						
.equ adr_OW_ROM_SEARCH_A			=0x0060+618		; "BIT A" (SEARCH)				
.equ adr_OW_RX_ROM_COMMAND			=0x0060+620		; ROM COMMAND 					
.equ adr_OW_RX_ROM_COMMAND_FERTIG	=0x0060+622		; ROM COMMAND abgearbeitet		
.equ adr_OW_RX_FUNCTION_COMMAND		=0x0060+624		; FUNCTION COMMAND				
.equ adr_OW_RX_FUNCTION_COMMAND_FERTIG=0x0060+626	; FUNCTION COMMAND abgearbeitet	
.equ adr_OW_ROM_ID_BEREICH8			=0x0060+630		; 8 BYTE ROM-ID SLAVE				
.equ adr_OW_SCRATCHPAD_A_BEREICH	=0x0060+650		; 32BYTE SCRATCHPAD (256 Bit)		
.equ adr_OW_SCRATCHPAD_B_BEREICH	=0x0060+700		; 32BYTE SCRATCHPAD (256 Bit)		
.equ adr_OW_SCRATCHPAD_C_BEREICH	=0x0060+750		; 32BYTE SCRATCHPAD (256 Bit)		

.equ adr_SRAM_CHECK					=0x0060+943		; SRAM CHECK muss immer "S" sein	
; REGISTER										

.def DUMMY_R0	= R0	; wichtige Hilfsregister (mit Vorsicht verwenden !)
.def DUMMY_R1	= R1	; wichtige Hilfsregister (mit Vorsicht verwenden !)


;.def OW_STATUS	= R10	; 1-WIRE-STATUS										

.def NULL 		= R12
.def EINS 		= R14
.def VOLL		= R15

.def temp 		= R16
.def temp1	 	= R17
.def temp2 		= R18
.def temp3 		= R19
.def temp4 		= R20
.def temp5 		= R21
.def temp6 		= R22
.def temp7 		= R23
.def temp8 		= R24
.def temp9 		= R25





.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT			; IRQ0 Handler                      					
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF   ; Timer0 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	


; Unkorrekten Interrupt Aufruf 						
INTERRUPT_ERROR:
	rjmp INTERRUPT_ERROR

RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM lschen										
	rcall SRAM_CLEAR
; PORT B initialisieren								
	ldi temp, 0b00000000	; 
	out DDRB, temp
; PORT B setzen										
	out PORTB, temp				
; PORT C initialisieren								
	ldi temp, 0b00000000	; 
	out DDRC, temp
; PORT C setzen										
	out PORTC, temp				
; PORT D initialisieren								
	ldi temp, 0b00000000	; 
	out DDRD, temp
; PORT D setzen										
	out PORTD, temp				
; WDR_INITIALISIERUNG								
	rcall WDR_INITIALISIERUNG
; NEUSTART_MIT_WDR									
	rcall NEUSTART_MIT_WDR
; 	LCD_INITIALISIERUNG								
	rcall LCD_INITIALISIERUNG
; 	LCD BEGRUESSUNG									
	rcall ANZEIGE_BEGRUESSUNG
; 1-WIRE_INITIALISIERUNG							
	rcall ONE_WIRE_INITIALISIERUNG
; 1-WIRE-EXTERNER_INTERUPT_INITIALISIEREN			
	rcall EXTERNER_INTERUPT_INITIALISIEREN
; 1-WIRE-EXTERNER_INTERRUPT_AKTIVIEREN				
	rcall EXTERNER_INTERRUPT_AKTIVIEREN						
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; VOLL				
	ldi temp,255
	mov VOLL,temp
	; SRAM CHECK		
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
; 	PROGRAMM-CHECK									
	rcall PROGRAMM_CHECK

; Interrupt															
	out GIFR,VOLL			; eventuelle Pegelwechsel ignorieren	
	SEI						; interrupts freigeben					
	; GRN				
	cbi (OW_LED_PORT),LED_OW_GRUEN

; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR									; Watch-Dog-RESET	
	rcall ANZEIGE_ERROR_CODE
	rcall ANZEIGE_WARTUNG
; ------------------------------------------------------------------------------
	rcall PROGRAMM_CHECK
	rcall wait_100ms
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "SL_1WIRE_ALLGEMEIN.asm"		; 1-wire-INTERRUPTS				
.include "SL_1WIRE_ROM_COMMAND.asm"		; 1-wire ROM-COMMANDS			
.include "SL_1WIRE_FUNCTION_COMMAND.asm"; 1-wire FUNKTION-COMMANDS		
.include "SL_LCD_ini.asm"				; LCD  Initialisierung 2/4x16/20
.include "SL_LCD.asm"					; LCD allgemein					
.include "SL_SONSTIGES.asm"				; sonstiges						
.include "SL_WAIT_16MHZ.asm"			; 16 MHz						
; ##############################################################################
; ##############################################################################
; ##############################################################################
TEXT_BEGRUESSUNG_1:
.db "1-WIRE-SLAVE ",0
TEXT_BEGRUESSUNG_2:
.db "ATmega8",0


